@@ -6,18 +6,18 @@
  *
  * This file is part of mpv.
  *
- * mpv is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * mpv is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with mpv.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,7 +30,7 @@
 
 #include <libavutil/intfloat.h>
 #include <libavutil/common.h>
-#include "talloc.h"
+#include "mpv_talloc.h"
 #include "ebml.h"
 #include "stream/stream.h"
 #include "common/msg.h"
@@ -73,56 +73,6 @@ uint32_t ebml_read_id(stream_t *s)
     return id;
 }
 
-/*
- * Read a variable length unsigned int.
- */
-uint64_t ebml_read_vlen_uint(bstr *buffer)
-{
-    int i, j, num_ffs = 0, len_mask = 0x80;
-    uint64_t num;
-
-    if (buffer->len == 0)
-        return EBML_UINT_INVALID;
-
-    for (i = 0, num = buffer->start[0]; i < 8 && !(num & len_mask); i++)
-        len_mask >>= 1;
-    if (i >= 8)
-        return EBML_UINT_INVALID;
-    j = i + 1;
-    if ((int) (num &= (len_mask - 1)) == len_mask - 1)
-        num_ffs++;
-    if (j > buffer->len)
-        return EBML_UINT_INVALID;
-    for (int n = 0; n < i; n++) {
-        num = (num << 8) | buffer->start[n + 1];
-        if ((num & 0xFF) == 0xFF)
-            num_ffs++;
-    }
-    if (j == num_ffs)
-        return EBML_UINT_INVALID;
-    buffer->start += j;
-    buffer->len -= j;
-    return num;
-}
-
-/*
- * Read a variable length signed int.
- */
-int64_t ebml_read_vlen_int(bstr *buffer)
-{
-    uint64_t unum;
-    int l;
-
-    /* read as unsigned number first */
-    size_t len = buffer->len;
-    unum = ebml_read_vlen_uint(buffer);
-    if (unum == EBML_UINT_INVALID)
-        return EBML_INT_INVALID;
-    l = len - buffer->len;
-
-    return unum - ((1 << ((7 * l) - 1)) - 1);
-}
-
 /*
  * Read: element content length.
  */
@@ -150,6 +100,25 @@ uint64_t ebml_read_length(stream_t *s)
     return len;
 }
 
+
+/*
+ * Read a variable length signed int.
+ */
+int64_t ebml_read_signed_length(stream_t *s)
+{
+    uint64_t unum;
+    int l;
+
+    /* read as unsigned number first */
+    uint64_t offset = stream_tell(s);
+    unum = ebml_read_length(s);
+    if (unum == EBML_UINT_INVALID)
+        return EBML_INT_INVALID;
+    l = stream_tell(s) - offset;
+
+    return unum - ((1LL << ((7 * l) - 1)) - 1);
+}
+
 /*
  * Read the next element as an unsigned int.
  */
@@ -358,7 +327,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
 {
     assert(type->type == EBML_TYPE_SUBELEMENTS);
     assert(level < 8);
-    MP_DBG(ctx, "%.*sParsing element %s\n", level, "       ", type->name);
+    MP_TRACE(ctx, "%.*sParsing element %s\n", level, "       ", type->name);
 
     char *s = target;
     uint8_t *end = data + size;
@@ -371,7 +340,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         if (len > end - p)
             goto past_end_error;
         if (len < 0) {
-            MP_DBG(ctx, "Error parsing subelement id\n");
+            MP_ERR(ctx, "Error parsing subelement id\n");
             goto other_error;
         }
         p += len;
@@ -379,7 +348,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         if (len > end - p)
             goto past_end_error;
         if (len < 0) {
-            MP_DBG(ctx, "Error parsing subelement length\n");
+            MP_ERR(ctx, "Error parsing subelement length\n");
             goto other_error;
         }
         p += len;
@@ -399,7 +368,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         if (length > end - p) {
             if (field_idx >= 0 && type->fields[field_idx].desc->type
                 != EBML_TYPE_SUBELEMENTS) {
-                MP_DBG(ctx, "Subelement content goes "
+                MP_ERR(ctx, "Subelement content goes "
                        "past end of containing element\n");
                 goto other_error;
             }
@@ -412,7 +381,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         continue;
 
     past_end_error:
-        MP_DBG(ctx, "Subelement headers go past end of containing element\n");
+        MP_ERR(ctx, "Subelement headers go past end of containing element\n");
     other_error:
         ctx->has_errors = true;
         end = startp;
@@ -468,20 +437,20 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         int len;
         uint32_t id = ebml_parse_id(data, end - data, &len);
         if (len < 0 || len > end - data) {
-            MP_DBG(ctx, "Error parsing subelement\n");
+            MP_ERR(ctx, "Error parsing subelement\n");
             break;
         }
         data += len;
         uint64_t length = ebml_parse_length(data, end - data, &len);
         if (len < 0 || len > end - data) {
-            MP_DBG(ctx, "Error parsing subelement length\n");
+            MP_ERR(ctx, "Error parsing subelement length\n");
             break;
         }
         data += len;
         if (length > end - data) {
             // Try to parse what is possible from inside this partial element
             length = end - data;
-            MP_DBG(ctx, "Next subelement content goes "
+            MP_ERR(ctx, "Next subelement content goes "
                    "past end of containing element, will be truncated\n");
         }
         int field_idx = -1;
@@ -491,16 +460,17 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
                 break;
             }
         if (field_idx < 0) {
-            if (id == 0xec)
-                MP_DBG(ctx, "%.*sIgnoring Void element "
-                       "size: %"PRIu64"\n", level+1, "        ", length);
-            else if (id == 0xbf)
-                MP_DBG(ctx, "%.*sIgnoring CRC-32 "
-                       "element size: %"PRIu64"\n", level+1, "        ",
-                       length);
-            else
+            if (id == 0xec) {
+                MP_TRACE(ctx, "%.*sIgnoring Void element "
+                         "size: %"PRIu64"\n", level+1, "        ", length);
+            } else if (id == 0xbf) {
+                MP_TRACE(ctx, "%.*sIgnoring CRC-32 "
+                         "element size: %"PRIu64"\n", level+1, "        ",
+                         length);
+            } else {
                 MP_DBG(ctx, "Ignoring unrecognized "
                        "subelement. ID: %x size: %"PRIu64"\n", id, length);
+            }
             data += length;
             continue;
         }
@@ -509,27 +479,27 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
         bool multiple = fd->multiple;
         int *countptr = (int *) (s + fd->count_offset);
         if (*countptr >= num_elems[field_idx]) {
-            // Shouldn't happen with on any sane file without bugs
-            MP_ERR(ctx, "Too many subelems?\n");
+            // Shouldn't happen on any sane file without bugs
+            MP_ERR(ctx, "Too many subelements.\n");
             ctx->has_errors = true;
             data += length;
             continue;
         }
         if (*countptr > 0 && !multiple) {
-            MP_DBG(ctx, "Another subelement of type "
-                   "%x %s (size: %"PRIu64"). Only one allowed. Ignoring.\n",
-                   id, ed->name, length);
+            MP_WARN(ctx, "Another subelement of type "
+                    "%x %s (size: %"PRIu64"). Only one allowed. Ignoring.\n",
+                    id, ed->name, length);
             ctx->has_errors = true;
             data += length;
             continue;
         }
-        MP_DBG(ctx, "%.*sParsing %x %s size: %"PRIu64
-               " value: ", level+1, "        ", id, ed->name, length);
+        MP_TRACE(ctx, "%.*sParsing %x %s size: %"PRIu64
+                 " value: ", level+1, "        ", id, ed->name, length);
 
         char *fieldptr = s + fd->offset;
         switch (ed->type) {
         case EBML_TYPE_SUBELEMENTS:
-            MP_DBG(ctx, "subelements\n");
+            MP_TRACE(ctx, "subelements\n");
             char *subelptr;
             if (multiple) {
                 char *array_start = (char *) *(generic_struct **) fieldptr;
@@ -548,29 +518,29 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
                 subelptr = (fieldtype *) fieldptr
             GETPTR(uintptr, uint64_t);
             if (length < 1 || length > 8) {
-                MP_DBG(ctx, "uint invalid length %"PRIu64"\n", length);
+                MP_ERR(ctx, "uint invalid length %"PRIu64"\n", length);
                 goto error;
             }
             *uintptr = ebml_parse_uint(data, length);
-            MP_DBG(ctx, "uint %"PRIu64"\n", *uintptr);
+            MP_TRACE(ctx, "uint %"PRIu64"\n", *uintptr);
             break;
 
         case EBML_TYPE_SINT:;
             int64_t *sintptr;
             GETPTR(sintptr, int64_t);
             if (length > 8) {
-                MP_DBG(ctx, "sint invalid length %"PRIu64"\n", length);
+                MP_ERR(ctx, "sint invalid length %"PRIu64"\n", length);
                 goto error;
             }
             *sintptr = ebml_parse_sint(data, length);
-            MP_DBG(ctx, "sint %"PRId64"\n", *sintptr);
+            MP_TRACE(ctx, "sint %"PRId64"\n", *sintptr);
             break;
 
         case EBML_TYPE_FLOAT:;
             double *floatptr;
             GETPTR(floatptr, double);
             if (length != 0 && length != 4 && length != 8) {
-                MP_DBG(ctx, "float invalid length %"PRIu64"\n", length);
+                MP_ERR(ctx, "float invalid length %"PRIu64"\n", length);
                 goto error;
             }
             *floatptr = ebml_parse_float(data, length);
@@ -585,7 +555,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
             char **strptr;
             GETPTR(strptr, char *);
             *strptr = talloc_strndup(ctx->talloc_ctx, data, length);
-            MP_DBG(ctx, "string \"%s\"\n", *strptr);
+            MP_TRACE(ctx, "string \"%s\"\n", *strptr);
             break;
 
         case EBML_TYPE_BINARY:;
@@ -597,7 +567,7 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
             GETPTR(binptr, struct bstr);
             binptr->start = data;
             binptr->len = length;
-            MP_DBG(ctx, "binary %zd bytes\n", binptr->len);
+            MP_TRACE(ctx, "binary %zd bytes\n", binptr->len);
             break;
 
         case EBML_TYPE_EBML_ID:;
@@ -605,10 +575,10 @@ static void ebml_parse_element(struct ebml_parse_ctx *ctx, void *target,
             GETPTR(idptr, uint32_t);
             *idptr = ebml_parse_id(data, end - data, &len);
             if (len != length) {
-                MP_DBG(ctx, "ebml_id broken value\n");
+                MP_ERR(ctx, "ebml_id broken value\n");
                 goto error;
             }
-            MP_DBG(ctx, "ebml_id %x\n", (unsigned)*idptr);
+            MP_TRACE(ctx, "ebml_id %x\n", (unsigned)*idptr);
             break;
         default:
             abort();
@@ -631,6 +601,10 @@ int ebml_read_element(struct stream *s, struct ebml_parse_ctx *ctx,
                    "- partial or corrupt file?\n");
         return -1;
     }
+    if (length == EBML_UINT_INVALID) {
+        MP_MSG(ctx, msglevel, "EBML element with unknown length - unsupported\n");
+        return -1;
+    }
     if (length > 1000000000) {
         MP_MSG(ctx, msglevel, "Refusing to read element over 100 MB in size\n");
         return -1;
